// Author: forrestfunk81@noctrl.eu
// SystemInfo gnome extension - version 2
// Distributed under the terms of the GNU General Public License v3

const St = imports.gi.St;
const GLib = imports.gi.GLib; 
const Mainloop = imports.mainloop;
const Main = imports.ui.main;
const Lang = imports.lang;
const PanelMenu = imports.ui.panelMenu;
const Gettext = imports.gettext;
const Util = imports.misc.util;
const GTop = imports.gi.GTop;
const PopupMenu = imports.ui.popupMenu; 
 
const _ = Gettext.gettext;
 
function SystemInfo() {
    this._init();
}

SystemInfo.prototype = {
	__proto__: PanelMenu.SystemStatusButton.prototype,
		
	// init SystemInfo
	_init: function() {
		PanelMenu.SystemStatusButton.prototype._init.call(this, 'systeminfo');
		
        this.statusLabel = new St.Label({
            text: "--",
            style_class: "systeminfo-label"
        });
        
        // destroy all previously created children, and add our statusLabel
        this.actor.get_children().forEach(function(c) {
            c.destroy();
        });
        this.actor.add_actor(this.statusLabel);
        this.sensorsPath = this._detectSensors();  
        this.nvidiaSettingsPath = this._detectNvidiaSettings();
        
        cpuUse = 0;
        memUse = 0;
        temp = 0;
        
        this._initValues();
        this._updateUI();
        
        //update every 2 seconds
        event = GLib.timeout_add_seconds(0, 2, Lang.bind(this, function () {
            this._updateUI();
            return true;
        }));
	},
	
	// call updateValues and update the ui
	_updateUI: function() {
		this._updateValues();
		
		cpuText = "0" + cpuUse ;
		cpuText = cpuText.substr(cpuText.length - 2, cpuText.length);		
		memText = memUse + "";
		
		this.statusLabel.set_text("CPU: " + cpuText + " % / Mem: " + memText + " %"); 
	},

	// initialize values
	_initValues: function() { 
        this._prev = new GTop.glibtop_cpu;
        GTop.glibtop_get_cpu(this._prev);
        
        this.mem = new GTop.glibtop_mem;
	},
	
	// update cpu and mem values
    _updateValues: function() { 
    	// cpu shit
        let cpu = new GTop.glibtop_cpu; 
        GTop.glibtop_get_cpu(cpu);
        let total = cpu.total - this._prev.total;
        let user = cpu.user - this._prev.user;
        let sys = cpu.sys - this._prev.sys;
        let iowait = cpu.iowait - this._prev.iowait;
        let idle = cpu.idle - this._prev.idle; 
        
        cpuUse = Math.round(100.0 * (user+sys+iowait) / total);
        this._prev = cpu;
        
        // mem
        GTop.glibtop_get_mem(this.mem);
        memUse =  Math.round(100.0 * this.mem.used / this.mem.total);         
        
        this.menu.box.get_children().forEach(function(c) {
            c.destroy()
        });
        let section = new PopupMenu.PopupMenuSection("SystemInfo");
        this.menu.addMenuItem(section);
        
        // graphics temperature
        if (this.nvidiaSettingsPath) {
        	let nvidia_output = GLib.spawn_command_line_sync(this.nvidiaSettingsPath)[1] + "";
        	let nvidia_lines = nvidia_output.split("\n");
        	let item = new PopupMenu.PopupMenuItem("Graphics temperature: " + nvidia_lines[0] +"\u1d3cC");
            section.addMenuItem(item); 
        }
                
        // temperatures and fan speeds
        if (this.sensorsPath){
	        let sensors_output = GLib.spawn_command_line_sync(this.sensorsPath)[1] + "";
	        let senses_lines=sensors_output.split("\n");
	        for(i = 0; i < senses_lines.length; i++) { 
	        	line = senses_lines[i];
	        	type=line.substr(9,line.length-9);
	            switch (type){
	            	case 'ISA adapter':
	            		for (j=i+1;;j++,i++){
	                        //continue only if line exists and isn't adapter
	                        if(senses_lines[j] && !this._isAdapter(senses_lines[j])){
	                            if(senses_lines[j].substr(0,4)=='Core'){
	                                senses_lines[j]=senses_lines[j].replace(/\s/g, "");
	                                temp = parseFloat(senses_lines[j].substr(7,4));
	                                
	                                let item = new PopupMenu.PopupMenuItem("Temperature: " + temp +"\u1d3cC");
	                                section.addMenuItem(item); 
	                            }
	                        }
	                        else break;
	            		} 
	            		break;
	        		case 'Virtual device':
	        			//starting from the next line, loop, also increase the outer line counter i
	                    for (j=i+1;;j++,i++){
	                        //continue only if line exists and isn't adapter
	                        if(senses_lines[j] && !this._isAdapter(senses_lines[j])){
	                            if(senses_lines[j].substr(0,5)=='temp1'){
	                                //remove all space characters
	                                senses_lines[j]=senses_lines[j].replace(/\s/g, "");
	                                temp = parseFloat(senses_lines[j].substr(7,4));
	                                
	                                let item = new PopupMenu.PopupMenuItem("Temperature: " + temp+"\u1d3cC");
	                                section.addMenuItem(item);    
	                            }
	                        }
	                        else break;
	                    }
	                    break;
	                case 'ACPI interface':
	                    //starting from the next line, loop, also increase the outer line counter i
	                    for (j=i+1;;j++,i++){
	                        //continue only if line exists and isn't adapter
	                        if(senses_lines[j] && !this._isAdapter(senses_lines[j])){
	                            if(senses_lines[j].substr(0,8)=='CPU Temp'){
	                                senses_lines[j]=senses_lines[j].replace(/\s/g, "");                                
	                                let item = new PopupMenu.PopupMenuItem("CPU Temperature: " + senses_lines[j].substr(16,4) +"\u1d3cC"); 
	                                section.addMenuItem(item); 
	                            }
	                            if(senses_lines[j].substr(0,8)=='MB Tempe'){
	                                senses_lines[j]=senses_lines[j].replace(/\s/g, "");                                  
	                                let item = new PopupMenu.PopupMenuItem("MB Temperature: " + senses_lines[j].substr(15,4) +"\u1d3cC");
	                                section.addMenuItem(item); 
	                            }
	                            if(senses_lines[j].substr(0,13)=='CPU FAN Speed'){
	                                senses_lines[j]=senses_lines[j].replace(/\s/g, "");                                  
	                                let item = new PopupMenu.PopupMenuItem("CPU Fan Speed: " + senses_lines[j].substr(12,4)+" RPM");
	                                section.addMenuItem(item); 
	                            }
	                            if(senses_lines[j].substr(0,13)=='CHASSIS FAN S'){                              
	                                let item = new PopupMenu.PopupMenuItem("Chassis Fan Speed: " + senses_lines[j].substr(21,4) +" RPM");
	                                section.addMenuItem(item); 
	                            }
	                            if(senses_lines[j].substr(0,13)=='CHASSIS FAN 2'){                                  
	                                let item = new PopupMenu.PopupMenuItem("Chassis Fan Speed: " + senses_lines[j].substr(21,4) +" RPM");
	                                section.addMenuItem(item); 
	                            }
	                            
	                        }
	                        else break;
	                    }
	                    break;
	                case 'PCI adapter':
	                    if (senses_lines[i-1].substr(0,6)=='k10tem' || senses_lines[i-1].substr(0,6)=='k8temp'){
	                        //starting from the next line, loop, also increase the outer line counter i
	                        for (j=i+1;;j++,i++){
	                            //continue only if line exists and isn't adapter
	                            if(senses_lines[j] && !this._isAdapter(senses_lines[j])){
	                                if(senses_lines[j].substr(0,5)=='temp1'){
	                                    senses_lines[j]=senses_lines[j].replace(/\s/g, "");
	                                    temp=parseFloat(senses_lines[j].substr(7,4));
	                                    
	                                    let item = new PopupMenu.PopupMenuItem("Temperature: " + temp+"C");
	                                    section.addMenuItem(item); 
	                                }
	                            }
	                            else break;
	                        }
	                    }
	                    break;
	                default:
	                    break;
	            }
	        }
        }
    },
	
    _isAdapter: function(line){
        if(line.substr(0, 8)=='Adapter:') {
            return true;
        }
        return false;
    },
    
    
    
    _detectSensors: function(){
        //detect if sensors is installed
        let ret = GLib.spawn_command_line_sync("which sensors");
        if ( (ret[0]) && (ret[3] == 0) ) {//if yes
            return ret[1].toString().split("\n", 1)[0];//find the path of the sensors
        }
        return null;
    },
    
    _detectNvidiaSettings: function(){
        //detect if nvidia-settings is installed
        let ret = GLib.spawn_command_line_sync("which nvidia-settings");
        if ( (ret[0]) && (ret[3] == 0) ) {
            return ret[1].toString().split("\n", 1)[0] + "  -q gpucoretemp -t";
        }
        return null;
    },
    
	_onDestroy: function() {}
};

//for debugging
function debug(a){
    global.log(a);
    Util.spawn(['echo',a]);
}

let _panelText;
let event=null;

function init() {
	_panelText = new SystemInfo();
}

function enable() {
	_panelText = new SystemInfo();
	Main.panel.addToStatusArea('systeminfo', _panelText);
}

function disable(){
	_panelText.destroy();
    Mainloop.source_remove(event);
    _panelText = null;
};